#!/bin/bash
set -ex

DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"

echo "Starting supervisor"
/usr/bin/supervisord -n -c /etc/supervisord.conf &
SUPERVISOR_PID=$!

source service_helpers.sh

function load_secrets() {
  wait_for_secrets_manager

  cloudfront_secret_string=$(<"$DIR/localdev/secrets/cloudfront_key_pair.json")
  sensor_secret_string=$(<"$DIR/localdev/secrets/sensor_key_pair.json")

  declare -A secrets_map
  secrets_map["${CLOUDFRONT_KEY_PAIR_SECRET_NAME}"]=$cloudfront_secret_string
  secrets_map["${SENSOR_KEY_PAIR_SECRET_NAME}"]=$sensor_secret_string

  for i in "${!secrets_map[@]}"
  do
    # Ensure the secret doesn't exist before we try to create it. I would have preferred to create then update if necessary,
    # but there's a localstack bug causing a 500 on any update-secret operation.
    aws --region=us-east-1 --endpoint-url="${SECRETS_MANAGER_ENDPOINT}" secretsmanager delete-secret --secret-id="${i}" --force-delete-without-recovery || true

    # Per the aws cli documentation, delete-secret initiates a background job that may not execute immediately.
    # Sleeping briefly before trying to re-create the secret.
    sleep 1
    aws --region=us-east-1 --endpoint-url="${SECRETS_MANAGER_ENDPOINT}" secretsmanager create-secret --name="${i}" --secret-string "${secrets_map[$i]}"
  done
}

function load_sqs() {
  wait_for_sqs

  aws --region=us-east-1 --endpoint-url="${SQS_ENDPOINT}" sqs create-queue --queue-name "${SQS_QUEUE_NAME}" --attributes file://"${DIR}"/localdev/sqs/attributes.json

}

function load_lambda() {
  wait_for_lambda

  # Zip Binary
  zip ruleset-cache-evict.zip ruleset_cache_evict

  listFuncs=$(aws --region=us-east-1 --endpoint-url="${LAMBDA_ENDPOINT}" lambda list-functions)

  # Check if evict lambda exists, if it does delete so we can recreate without error.
  if [[ "$listFuncs" == *"${RULESET_CACHE_EVICT_NAME}"* ]]; then
    aws --region=us-east-1 --endpoint-url="${LAMBDA_ENDPOINT}" lambda delete-function --function-name "${RULESET_CACHE_EVICT_NAME}"
  fi

  create_ruleset_cache_evict_lambda
}

function create_ruleset_cache_evict_lambda() {
  aws --region=us-east-1 --endpoint-url="${LAMBDA_ENDPOINT}" lambda create-function \
    --function-name "${RULESET_CACHE_EVICT_NAME}" \
    --runtime go1.x \
    --zip-file fileb://ruleset-cache-evict.zip \
    --handler ruleset_cache_evict \
    --role testrole \
    --environment "Variables={REDIS_ADDRESS=${REDIS_ADDRESS},CLUSTER_NAME=${CLUSTER_NAME},AWS_REGION=${AWS_REGION},AWS_ACCESS_KEY_ID=${AWS_ACCESS_KEY_ID},AWS_SECRET_ACCESS_KEY=${AWS_SECRET_ACCESS_KEY},DISABLE_AWS_SSL=${DISABLE_AWS_SSL},S3_FORCE_PATH_STYLE=${S3_FORCE_PATH_STYLE}}"

  # Create S3 trigger
  aws s3api --region=us-east-1 --endpoint-url="${S3_ENDPOINT}" put-bucket-notification-configuration --bucket "${S3_RULESET_BUCKET}" --notification-configuration file://"${DIR}"/localdev/s3/bucket-trigger.json
}

function load_s3() {
  wait_for_s3
  aws  --region=us-east-1 --endpoint-url="${S3_ENDPOINT}"  s3 mb s3://"${S3_RULESET_BUCKET}"

  # Create S3 bucket for testing external urls
  aws  --region=us-east-1 --endpoint-url="${S3_ENDPOINT}"  s3 mb "s3://${S3_EXTERNAL_TEST}"
  aws  --region=us-east-1 --endpoint-url="${S3_ENDPOINT}"  s3 cp "${DIR}/localdev/resources/test.txt" "s3://${S3_EXTERNAL_TEST}/test.txt"
}

# Add secrets to localstack secrets manager
load_secrets

# Add SQS queue
load_sqs

# Add s3 bucket for ruleset generator
load_s3

# Add Lambda and create event source mapping for localdev sqs
load_lambda

tail -qF /tmp/localstack_infra.log /tmp/localstack_infra.err

wait $SUPERVISOR_PID

