#!/usr/bin/env bash

set -ex

# Resetting the current working directory to the repo root if the script is run from it's location
if [[ "$(basename ${PWD})" == "tests" ]]; then
  cd ../../
fi

CURRENT_DIR_FULL_PATH=${PWD}
SKIP_BUILD=${SKIP_BUILD:-false}
RUN_TESTS_ONLY=${RUN_TESTS_ONLY:-false}  # Only builds and runs the integration tests. Assumes services are already up locally
TEST_REPORTS_DIR=${TEST_REPORTS_DIR:-"test/test-reports"}
DOCKER_LOGS_DIR=${DOCKER_LOGS_DIR:-"docker/logs"}
PARALLEL=${PARALLEL:-true}
CACHE_TESTS=${CACHE_TESTS:-false}
CLOUDFRONT_SIGNER=${CLOUDFRONT_SIGNER:-true}

# shellcheck source=.
source "${CURRENT_DIR_FULL_PATH}/helpers/service_helpers.sh"
# shellcheck source=.
source "${CURRENT_DIR_FULL_PATH}/helpers/pretty_print.sh"

printp "RUNNING INTEGRATION TESTS"

wait_for_service() {
  : ${SERVICE_NAME:?"Missing environment variable"}
    TIMEOUTS=0
    until [ "$( docker ps -a | grep "$SERVICE_NAME" | wc -l )" -gt 0 ] || [[ ${TIMEOUTS} -eq 5 ]]; do echo "Waiting for $SERVICE_NAME to come up..."; sleep 5; ((TIMEOUTS+=1)); done
}

if [[ ${RUN_TESTS_ONLY} != "true" ]]; then
    # Pull everything except the content pacing service and the content generator
    make inverse-pull-full-stack-integration-local

    # Build cps service
    if [[ ${SKIP_BUILD} != "true" ]]; then
        make docker-build build-content-generator
    fi

    SERVICES="postgres redis rms-postgres redis-cps cps-mockaws redis_auth mockaws access-mockaws"
    docker-compose -f ./docker/docker-compose-integration-local.yml up -d $SERVICES

    # This should be handled by the depends attribute of the services in docker-compose.yaml!
    for SERVICE in $SERVICES; do
      SERVICE_NAME="$SERVICE" wait_for_service
    done


    # DREC Content Generator needs auth stack to be up since it calls RMS through the gateway
    docker-compose -f ./docker/docker-compose-integration-local.yml up -d appservices psc.auth device-gateway
    sleep 5
    appservices_container=$(docker ps --format '{{.Names}}' --filter "name=appservices")
    retry "docker exec ${appservices_container} curl -s http://appservices:8080/appservices/healthCheck"

    docker-compose -f ./docker/docker-compose-integration-local.yml up -d psc.access
    sleep 5
    retry "docker exec ${appservices_container} curl -s http://psc.access:8080/access/v1/healthCheck"

    docker-compose -f ./docker/docker-compose-integration-local.yml up -d psc.rules
    sleep 5
    rules_container=$(docker ps --format '{{.Names}}' --filter "name=psc.rules")
    retry "docker exec ${rules_container} curl -s http://localhost:8080/rules/v1/health_check"

    cps_mockaws_container=$(docker ps --format '{{.Names}}' --filter "name=cps-mockaws")
    wait_for_service "AWS_ACCESS_KEY_ID=localdev AWS_SECRET_ACCESS_KEY=localdev docker exec ${cps_mockaws_container} aws --endpoint-url=http://localhost:4572 s3api list-buckets | grep localdev"

    # Start up the rest of the containers
    docker-compose -f ./docker/docker-compose-integration-local.yml up -d --no-recreate
    sleep 5
    retry "docker exec ${appservices_container} curl -v http://content-pacing:8030/content_pacing/health_check"
fi

# SETUP TEST CONTAINER AND TEST REPORTING
# Use timestamp for local, COMMIT hash for gitlab
if [[ -z "$CI_COMMIT_SHA" ]]; then
    CONTAINER_NAME="test-integration-$(date +%s)"
    TEST_REPORT_NAME="test_report_$(date +%s).xml"
    TEST_REPORT_FULL_PATH="${CURRENT_DIR_FULL_PATH}/${TEST_REPORTS_DIR}"
    DOCKER_LOGS_FULL_PATH="${CURRENT_DIR_FULL_PATH}/${DOCKER_LOGS_DIR}"
else
    CONTAINER_NAME="test-integration-${CI_COMMIT_SHA}-${CI_JOB_ID}"
    TEST_REPORT_NAME="${CI_COMMIT_SHA}_${CI_JOB_ID}_report.xml"
    TEST_REPORT_FULL_PATH="${CI_PROJECT_DIR}/${TEST_REPORTS_DIR}"
    DOCKER_LOGS_FULL_PATH="${CI_PROJECT_DIR}/${DOCKER_LOGS_DIR}"
fi

# Copy test results from the container if tests pass or fail
function copy_test_results {
    # Only copying the test report in the GitLab pipeline
    if [[ -n "$CI_COMMIT_SHA" ]]; then
        # Create a folder for test reports
        if [[ ! -d "${TEST_REPORT_FULL_PATH}" ]]; then
            mkdir -p "${TEST_REPORT_FULL_PATH}"
        fi

        printf "Copying test report %s to %s \r\n" "${TEST_REPORT_NAME}" "${TEST_REPORT_FULL_PATH}/${TEST_REPORT_NAME}"
        docker cp "${CONTAINER_NAME}:/${TEST_REPORTS_DIR}/${TEST_REPORT_NAME}" "${TEST_REPORT_FULL_PATH}/${TEST_REPORT_NAME}"
        docker rm "${CONTAINER_NAME}"
    fi
}

function copy_docker_logs {
    # Only copying the logs in the GitLab pipeline
    if [[ -n "$CI_COMMIT_SHA" ]]; then
        # Create a folder for docker logs
        if [[ ! -d "${DOCKER_LOGS_FULL_PATH}" ]]; then
            mkdir -p "${DOCKER_LOGS_FULL_PATH}"
        fi

        CONTAINER_NAMES="$(docker ps --format '{{.Names}}' --filter "name=docker-*")"
        echo CONTAINER_NAMES=$CONTAINER_NAMES
        for CONTAINER_NAME in $CONTAINER_NAMES; do
          docker logs ${CONTAINER_NAME} &> "${DOCKER_LOGS_FULL_PATH}/${CONTAINER_NAME}.log" || true
        done

#        cps_container=$(docker ps --format '{{.Names}}' --filter "name=docker_content-pacing")
#        rules_container=$(docker ps --format '{{.Names}}' --filter "name=docker_psc.rules")
#        rms_postgres_container=$(docker ps --format '{{.Names}}' --filter "name=docker_rms-postgres")
#        auth_container=$(docker ps --format '{{.Names}}' --filter "name=docker_psc.auth")
#        gateway_container=$(docker ps --format '{{.Names}}' --filter "name=docker_psc.gateway")
#        access_container=$(docker ps --format '{{.Names}}' --filter "name=docker_psc.access")
#        cps_mockaws_container=$(docker ps --format '{{.Names}}' --filter "name=docker_cps-mockaws")
#        ems_device_auth_container=$(docker ps --format '{{.Names}}' --filter "name=docker_ems-device-auth")
#        cg_ruleset_generator_container=$(docker ps --format '{{.Names}}' --filter "name=docker_cg-ruleset-generator")
#        cg_drec_ruleset_generator_container=$(docker ps --format '{{.Names}}' --filter "name=docker_cg-drec-ruleset-generator")
#
#        docker logs ${cps_container} &> "${DOCKER_LOGS_FULL_PATH}/content-pacing.log"
#        docker logs ${rules_container} &> "${DOCKER_LOGS_FULL_PATH}/psc-rules.log"
#        docker logs ${rms_postgres_container} &> "${DOCKER_LOGS_FULL_PATH}/rms-postgres.log"
#        docker logs ${auth_container} &> "${DOCKER_LOGS_FULL_PATH}/psc-auth.log"
#        docker logs ${gateway_container} &> "${DOCKER_LOGS_FULL_PATH}/psc-gateway.log"
#        docker logs ${access_container} &> "${DOCKER_LOGS_FULL_PATH}/psc-access.log"
#        docker logs ${cps_mockaws_container} &> "${DOCKER_LOGS_FULL_PATH}/cps-mockaws.log"
#        docker logs ${ems_device_auth_container} &> "${DOCKER_LOGS_FULL_PATH}/ems-device-auth.log"
#        docker logs ${cg_ruleset_generator_container} &> "${DOCKER_LOGS_FULL_PATH}/cg-ruleset-generator.log"
#        docker logs ${cg_drec_ruleset_generator_container} &> "${DOCKER_LOGS_FULL_PATH}/cg-drec-ruleset-generator.log"
    fi
}

function on_exit {
    copy_test_results
    copy_docker_logs
}

trap on_exit EXIT

# Build the test container
if [[ ${SKIP_BUILD} != "true" ]]; then
    docker build -t cbc-content-pytest test/pytest_integration/ -f ./docker/tests/Dockerfile.integration
fi

if [[ -z ${TEST_MARKERS} ]]
then
    TEST_MARKERS="not (scheduled_only or serial or env_smoke)"
else
    if [[ "${TEST_MARKERS}" == *"serial"* ]]; then
        TEST_MARKERS="${TEST_MARKERS} and not (scheduled_only or env_smoke)"
    else
        TEST_MARKERS="${TEST_MARKERS} and not (scheduled_only or serial or env_smoke)"
    fi
fi

if [[ ${CACHE_TESTS} == "true" ]]; then
    TEST_MARKERS="${TEST_MARKERS} and caching"
else
    TEST_MARKERS="${TEST_MARKERS} and not caching"
fi

if [[ "${PARALLEL}" == "true" ]]; then
    XDIST="--numprocesses=auto"
fi

# Display running containers to console
echo "------ RUNNING CONTAINERS ------"
docker ps

# Run the health check test first
docker run --network=docker_default --rm cbc-content-pytest pytest --pyargs cbc_content_pacing -k "health_check"

# Run the rest of the tests
echo "docker run -e \"CLOUDFRONT_SIGNER=${CLOUDFRONT_SIGNER}\" --name ${CONTAINER_NAME} --network=docker_default cbc-content-pytest pytest --pyargs cbc_content_pacing
    --durations=15 --junitxml=/${TEST_REPORTS_DIR}/${TEST_REPORT_NAME} ${XDIST} -m ${TEST_MARKERS}"

docker run -e "CLOUDFRONT_SIGNER=${CLOUDFRONT_SIGNER}" --name "${CONTAINER_NAME}" --network=docker_default cbc-content-pytest pytest --pyargs cbc_content_pacing \
  --collect-sguid --durations=15 --junitxml="/${TEST_REPORTS_DIR}/${TEST_REPORT_NAME}" ${XDIST} -m "${TEST_MARKERS}"
